@@ -1,17 +1,17 @@
 //! Enable loading the magic database files at runtime rather than embedding the GPLed database
 
+use std::collections::HashMap;
 use std::env::{split_paths, var_os};
 use std::ffi::OsString;
 use std::fs::{read, read_to_string};
 use std::path::{Path, PathBuf};
+use std::sync::OnceLock;
 
-use fnv::FnvHashMap;
-use once_cell::sync::OnceCell;
 use petgraph::prelude::DiGraph;
 
 use super::MagicRule;
-use crate::fdo_magic::ruleset;
 use crate::Mime;
+use crate::fdo_magic::ruleset;
 
 fn mime_path(base: &Path, filename: &str) -> PathBuf {
     base.join("mime").join(filename)
@@ -20,6 +20,12 @@ fn mime_path(base: &Path, filename: &str) -> PathBuf {
 fn search_paths(filename: &str) -> Vec<PathBuf> {
     let mut paths = Vec::new();
 
+    // If the TREE_MAGIC_DIR environment variable is set, use it directly
+    // and return so we just use specified directory.
+    if let Some(load_path) = var_os("TREE_MAGIC_DIR") {
+        return vec![PathBuf::from(load_path).join(filename)];
+    }
+
     let data_dirs = match var_os("XDG_DATA_DIRS") {
         Some(dirs) if !dirs.is_empty() => dirs,
         _ => OsString::from("/usr/local/share/:/usr/share/"),
@@ -60,17 +66,17 @@ fn load_concat_strings(filename: &str) -> String {
 }
 
 pub fn aliases() -> &'static str {
-    static ALIAS_STRING: OnceCell<String> = OnceCell::new();
+    static ALIAS_STRING: OnceLock<String> = OnceLock::new();
     ALIAS_STRING.get_or_init(|| load_concat_strings("aliases"))
 }
 
 pub fn subclasses() -> &'static str {
-    static SUBCLASS_STRING: OnceCell<String> = OnceCell::new();
+    static SUBCLASS_STRING: OnceLock<String> = OnceLock::new();
     SUBCLASS_STRING.get_or_init(|| load_concat_strings("subclasses"))
 }
 
-pub fn rules() -> Result<FnvHashMap<Mime, DiGraph<MagicRule<'static>, u32>>, String> {
-    static RUNTIME_RULES: OnceCell<Vec<Vec<u8>>> = OnceCell::new();
+pub fn rules() -> Result<HashMap<Mime, DiGraph<MagicRule<'static>, u32>>, String> {
+    static RUNTIME_RULES: OnceLock<Vec<Vec<u8>>> = OnceLock::new();
     let files = RUNTIME_RULES.get_or_init(load_xdg_shared_magic);
     ruleset::from_multiple(files)
 }
